// tissue p-system simulator
// copyright 2008, 2009 jamie twycross, jpt AT cs.nott.ac.uk
// released under GNU GPL version 3

#if HAVE_CONFIG_H
	#include <config.h>
#endif

#include <iostream>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <pthread.h>
#include <sys/types.h>
#include <unistd.h>
#include <signal.h>
#include <mcss/mcss.h>

using namespace std;
using namespace Simulation;

#ifndef WIN32
// timer thread data
struct TimerData {
	unsigned long max_time;
	pid_t pid;
};

// timer thread handler
void *timer(void *args)
{
	// get arguments
	struct TimerData *timer_data = (struct TimerData *) args;

	// sleep for specified runtime
	sleep(timer_data->max_time);

	// woken up, so kill mcss
	kill(timer_data->pid, SIGINT);

	pthread_exit(NULL);
}
#endif

int main(int argc, char **argv)
{
	#ifndef WIN32
	pthread_t timer_id;
	struct TimerData timer_data;
	#endif

	#ifdef HAVE_MPI
	// initialise mpi
	MPI_Init(&argc, &argv);
	atexit((void (*)()) MPI_Finalize);
	#endif

	// parse command line and read in parameters
	if(argc == 1)
		#ifdef HAVE_MPI
		fprintf(stderr, "mcss version %s (MPI)\n", VERSION);
		#else
		fprintf(stderr, "mcss version %s\n", VERSION);
		#endif
	SimulationParameters parameters;
	parameters.read(argc, argv);

	// create p-system from parameters specified in parameter file
	Psystem psystem(parameters);

	// dump model if not simulating
	if(parameters.dump) {
		char dump_file[PARAMETER_STR_MAX];
		char *ptr = strrchr(parameters.data_file, '.');
		*ptr = '\0';
		snprintf(dump_file, 1024, "%s.dump", parameters.data_file);
		psystem.dump(dump_file);
		exit(EXIT_SUCCESS);
	}

	#ifndef WIN32
	// set up timer thread if max_runtime specified
	if(parameters.max_runtime != 0.0) {
		timer_data.max_time = (unsigned int) parameters.max_runtime;
		timer_data.pid = getpid();
		if(pthread_create(&timer_id, NULL, timer, (void *) &timer_data)) {
			cerr << "error: unable to create timer thread\n";
			exit(EXIT_FAILURE);
		}
	}
	#endif

	// run simulation
	psystem.execute(parameters.max_time);

	#ifndef WIN32
	// stop timer thread
	if(parameters.max_runtime != 0.0)
		pthread_cancel(timer_id);
	#endif

	exit(EXIT_SUCCESS);
}
